package OpenInteract::SQLInstall::User;

# $Id: User.pm,v 1.1.1.1 2001/02/02 06:20:20 lachoy Exp $

# Do installation of SQL for this package

use strict;
use vars         qw( %HANDLERS );
use Data::Dumper qw( Dumper );
use SPOPS::Utility;

@OpenInteract::SQLInstall::User::ISA = qw( OpenInteract::SQLInstall );

use constant DEBUG => 0;

my %files = (
  tables   => [ 'user.sql' ],
  security => [ 'install_security.dat' ],
);

%HANDLERS = (
  create_structure => { '_default_' => [ 'create_structure', 
                                         { table_file_list => $files{tables} } ] },
  install_data     => { '_default_' => \&create_admin_user },
  install_security => { '_default_' => [ 'install_data',
                                         { data_file_list => $files{security} } ] },
);


# Create the admin user and give him/her a default password -- be sure
# to let the user know what this password is!

sub create_admin_user {
  my ( $class, $p ) = @_;
  my $status = { action => 'data', name => 'create_admin_user', ok => 1,
                 driver_name => $p->{config}->{db_info}->{driver_name} };
  my $user_class = $class->sql_class_to_website( 
                              $p->{config}->{website_name}, 
                              'OpenInteract::User' 
                   );
  warn " (create_admin_user): Trying to create user with class ($user_class)\n" if ( DEBUG );
  my $user = $user_class->new({ email      => $p->{config}->{admin_email},
                                login_name => 'superuser', 
                                first_name => 'Super',
                                last_name  => 'User', 
                                user_id    => 1 });
  my $password = SPOPS::Utility->generate_random_code( 8 );
  if ( $p->{config}->{login}->{crypt_password} ) {
    $user->{password} = $user->crypt_it( $password );
  }
  eval { $user->save( { is_add => 1, skip_security => 1, 
                        skip_cache => 1, skip_log => 1 } ) };
  if ( $@ ) {
    my $ei = SPOPS::Error->get;
    $status->{ok}  = 0;
    $status->{msg} = "Cannot create admin user!\nError: $@\nVerbose: $ei->{system_msg}";
    warn " (create_admin_user): Cannot create user! Error: ", Dumper( $ei ), "\n" if ( DEBUG );
  }
  else {
    $status->{msg} = "Created user (superuser)) with initial password: (($password)). **WRITE THIS PASSWORD DOWN!**";
    $status->{note} = "Administrator username: superuser\n" .
                      "Administrator password: $password\n" .
                      "**WRITE THIS INFORMATION DOWN**";
  } 
  return [ $status ];
}

1;

__END__

=pod

=head1 NAME

OpenInteract::SQLInstall::User - SQL installer for the base_user package

=head1 SYNOPSIS

=head1 DESCRIPTION

This SQLInstall subclass is a little different than many of the other
handlers in OpenInteract -- you B<must> track information that it
generates during the SQL installation process.

We do not want to ship OpenInteract with either a blank or otherwise
known superuser password. And we do not want to force users to type it
in during installation -- doing as much as possible to allow automated
installs is a good thing.

So when we install the SQL we create the superuser with a random
password and echo the password back to you.

=head1 BUGS 

=head1 TO DO

=head1 SEE ALSO

=head1 COPYRIGHT

Copyright (c) 2001 intes.net, inc.. All rights reserved.

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=head1 AUTHORS

Chris Winters <chris@cwinters.com>

=cut
